@@ -3,7 +3,7 @@ package Alien::Base::ModuleBuild;
 use strict;
 use warnings;
 
-our $VERSION = '0.015';
+our $VERSION = '0.016';
 $VERSION = eval $VERSION;
 
 use parent 'Module::Build';
@@ -127,6 +127,12 @@ __PACKAGE__->add_property( 'alien_msys' => 0 );
 # Alien packages that provide build dependencies
 __PACKAGE__->add_property( 'alien_bin_requires' => {} );
 
+# Alien packages that can/should be installed directory into the blib directory by the `./Build' command
+# rather than to the final location during the `./Build install` step.
+# This is the default when building an ACTIVESTATE_PPM_BUILD because AS does not do `./Build install` at
+# all when building a PPM
+__PACKAGE__->add_property( 'alien_stage_install' => $ENV{ACTIVESTATE_PPM_BUILD} ? 1 : 0 );
+
 ################
 #  ConfigData  #
 ################
@@ -259,7 +265,7 @@ sub ACTION_code {
     $self->SUPER::ACTION_code;
 
     # copy the compiled files into blib if running under blib scheme
-    $self->depends_on('alien_install') if $self->notes('alien_blib_scheme');
+    $self->depends_on('alien_install') if $self->notes('alien_blib_scheme') || $self->alien_stage_install;
   }
   
   my $module = $self->module_name;
@@ -395,7 +401,11 @@ sub ACTION_test {
 sub ACTION_install {
   my $self = shift;
   $self->SUPER::ACTION_install;
-  $self->depends_on('alien_install');
+  if($self->alien_stage_install) {
+    $self->alien_relocation_fixup if $self->alien_stage_install;
+  } else {
+    $self->depends_on('alien_install') unless $self->alien_stage_install;
+  }
 }
 
 sub ACTION_alien_install {
@@ -470,7 +480,8 @@ sub ACTION_alien_install {
     $self->SUPER::ACTION_install;
 
     # refresh the packlist
-    $self->alien_refresh_packlist( $self->alien_library_destination );
+    $self->alien_refresh_packlist( $self->alien_library_destination )
+      unless $self->alien_stage_install;
   }
 }
 
@@ -557,7 +568,7 @@ sub alien_library_destination {
 
   # send the library into the blib if running under the blib scheme
   my $lib_dir = 
-    $self->notes('alien_blib_scheme')
+    $self->notes('alien_blib_scheme') || $self->alien_stage_install
     ? File::Spec->catdir( $self->base_dir, $self->blib, 'lib' )
     : $self->install_destination('lib');
 
@@ -571,6 +582,7 @@ sub alien_library_destination {
 sub alien_detect_blib_scheme {
   my $self = shift;
 
+  return 0 if $self->alien_stage_install;
   return $ENV{ALIEN_BLIB} if defined $ENV{ALIEN_BLIB};
 
   # check to see if Alien::Base::ModuleBuild is running from blib.
@@ -662,6 +674,10 @@ sub _env_do_system {
     local $CWD;
     pop @CWD;
     
+    my $ldflags = $Config{ldflags};
+    $ldflags .= " -Wl,-headerpad_max_install_names"
+      if $^O eq 'darwin';
+    
     open my $fh, '>', 'config.site';
     print $fh "CC='$Config{cc}'\n";
     # -D define flags should be stripped because they are Perl
@@ -669,7 +685,7 @@ sub _env_do_system {
     print $fh "CFLAGS='", _filter_defines($Config{ccflags}), "'\n";
     print $fh "CPPFLAGS='", _filter_defines($Config{cppflags}), "'\n";
     print $fh "CXXFLAGS='", _filter_defines($Config{ccflags}), "'\n";
-    print $fh "LDFLAGS='$Config{ldflags}'\n";
+    print $fh "LDFLAGS='$ldflags'\n";
     close $fh;
   
     my $config ||= _shell_config_generate();
@@ -996,6 +1012,37 @@ sub alien_refresh_packlist {
   $packlist->write if $changed;
 }
 
+sub alien_relocation_fixup {
+  my($self) = @_;
+  
+  # so far relocation fixup is only needed on OS X
+  return unless $^O eq 'darwin';
+
+  my $dist_name = $self->dist_name;  
+  my $share = _catdir( $self->install_destination('lib'), qw/auto share dist/, $dist_name );
+  
+  require File::Find;
+  File::Find::find(sub {
+    if(/\.dylib$/)
+    {
+      # save the original mode and make it writable
+      my $mode = (stat $File::Find::name)[2];
+      chmod 0755, $File::Find::name unless -w $File::Find::name;
+      
+      my @cmd = (
+        'install_name_tool',
+        '-id' => $File::Find::name,
+        $File::Find::name,
+      );
+      print "+ @cmd\n";
+      system @cmd;
+      
+      # restore the original permission mode
+      chmod $mode, $File::Find::name;
+    }
+  }, $share);
+}
+
 sub _rscan_destdir {
   my($self, $dir, $pattern) = @_;
   my $destdir = $self->destdir;
